#ifndef COMPETE_H
#define COMPETE_H

#include <iostream> 
#include "strategy.h" 

#define DRAND()	  (((double) rand()) / ( RAND_MAX + 1.0))

class result {
public:
  int first_player;
  int second_player; 
};  


class Compete {
public:
  
  Compete(int _trials = 100,int _ptrials = 30); 
  
  result operator()(Strategy& pl1, Strategy& pl2); 
  void printOn() { print = true; }
  void printOff() { print = false; }
  void setTrials(int _trials,int _ptrials) { trials = _trials;  ptrials = _ptrials; }

  private:
  int rewards[2][2]; 
  int trials;
  int ptrials; 
  bool print; 
};  

Compete::Compete(int _trials,int _ptrials){
  rewards[COOPERATE][COOPERATE] = CC; 
  rewards[COOPERATE][DECEIVE] = CD;
  rewards[DECEIVE][COOPERATE] = DC; 
  rewards[DECEIVE][DECEIVE] = DD;
  
  trials = _trials;
  ptrials = _ptrials;
  print = false; 
} 

result Compete::operator()(Strategy& pl1, Strategy& pl2) {

  int res1, res2;
  int rec1[trials];
  int rec2[trials]; 



  pl1.reset(); pl2.reset(); // prepare for competition 

  result res; 
  res.first_player = 0;
  res.second_player = 0; 

  // compete 
  for (int i=0; i<trials; i++) {
   
    // ask for decision 
    rec1[i] = res1 = pl1(); 
    rec2[i] = res2 = pl2();
    

    // some strategies returns illegal value 
    if ((res1 != COOPERATE) && (res1 != DECEIVE)) {
      if (DRAND()>0.5)
	rec1[i] = res1 = COOPERATE;
      else
	rec1[i] = res1 = DECEIVE;
    } 

    if ((res2 != COOPERATE) && (res2 != DECEIVE)) {
      if (DRAND()>0.5)
	rec2[i] = res2 = COOPERATE;
      else
	rec2[i] = res2 = DECEIVE;
    } 


    // rewards
    pl1.reward( rewards[res1][res2] ); 
    pl2.reward( rewards[res2][res1] ); 
  
    res.first_player += rewards[res1][res2]; 
    res.second_player += rewards[res2][res1]; 
  }
  
  if (print) { 
    cout << pl1.getName() << " vs. " << pl2.getName() << endl; 
    cout <<"Player1:  ";
    for (int i=0; i<ptrials; i++) cout << ((rec1[i]) ? "D" : "C");
    cout << " => " << res.first_player << endl; 
    cout << "Player2:  "; 
    for (int i=0; i<ptrials; i++) cout << ((rec2[i]) ? "D" : "C"); 
    cout << " => " << res.second_player << endl << endl; 
  }
  
  return res; 
} 



#endif 
