#ifndef IND_STRATEGY_H
#define IND_STRATEGY_H

#include <eo> 
#include <ga.h>

#include "strategy.h" 

/*

Strategy defined by the 64 bits: 

format (Me, Oponent)
 
        0 ... cooperate 
	1 ... decieve 

 
     (CC) (CC) (CC) 0  0/1  
     (CC) (CC) (CD) 1  0/1  
     (CC) (CC) (DC) 2 
     (CC) (CC) (DD) 3 
      .
      .
      .
     (DD) (DD) (DD) 63 

the rightmost is the newest 

Record: 3 couples 
  (Me,Oponent) (Me, Oponent) (Me, Oponent) 
  
  saved as number  
  the lowest bits represent the newest records 
  

*/ 

using namespace std;


class IndiStrategy : public Strategy { 

public:

  // initialized by individual 
  IndiStrategy(const eoBit<double>& _bitmap) : bitmap(_bitmap), first(true), record(0) { 
    if (bitmap.size()!=70) 
      throw std::runtime_error("Individual's bitmap is not 70 bits long.");
  };

  virtual int operator()();  // returns the decision: true = decieve 
  //                       false = cooperate 
  
  virtual void reward(int _reward); // should be called after each trial 

  virtual void reset() { record = 0; first = true; }

  void print(); 

  virtual string getName() { return "EvolvedStrategy"; }
  
private: 
  const eoBit<double>& bitmap;   // reference to the bitmap defining the stragy (i.e. corresponding individual)   
  bool first; 
                          
  unsigned record; 
}; 




#endif
